@@ -17,7 +17,7 @@
 import functools
 import itertools
 
-from typing import Any, Dict, FrozenSet, Iterable, List, TypeVar, Callable,  Tuple, Optional, Sequence
+from typing import Any, Dict, FrozenSet, Iterable, Iterator, List, Mapping, TypeVar, Callable,  Tuple, Optional, Sequence
 
 _T = TypeVar("_T")
 _U = TypeVar("_U")
@@ -146,6 +146,13 @@ def union(*sets: FrozenSet[_T]) -> FrozenSet[_T]:
         return head.union(*tail)
     except ValueError:
         return frozenset()
+
+def invert_mapping(source: Mapping[_T, _V]) -> Dict[_V, _T]:
+    result = {
+        value: key  for (key, value) in source.items()
+    }
+    return result
+
 def find_subsets(s: FrozenSet[_T], n: int) -> FrozenSet[FrozenSet[_T]]:
     
     subsets_n = frozenset( # type: ignore
@@ -244,6 +251,10 @@ def all_equal(iterable: Iterable[Any]) -> bool:
     g = itertools.groupby(iterable)
     return next(g, True) and not next(g, False) # type: ignore
 
+
+    
+
+
 def list_unzip(iterable: Iterable[Tuple[_T, _U]]) -> Tuple[Sequence[_T], Sequence[_U]]:
     """Unzips an iterable of tuples of two elements, and returns a tuple of two lists,
     where the first contains all the first elements of the tuples and the latter the second
@@ -388,6 +399,17 @@ def filter_snd_none(fst_iter: Iterable[_T],
     zipped: Iterable[Tuple[_T, _U]] = filter(lambda x: x[1] is not None, zip(fst_iter, snd_iter)) # type: ignore
     return list_unzip(zipped)
 
+
+
+def zip_chain(iterable: Iterable[Tuple[Sequence[_T], Sequence[_U]]]) -> Iterator[Tuple[_T, _U]]:
+    combined = (zip(a,b) for (a,b) in iterable)
+    chained = itertools.chain.from_iterable(combined)
+    yield from chained
+
+def unzip_chain(iterable: Iterable[Tuple[Sequence[_T], Sequence[_U]]]) -> Tuple[Sequence[_T], Sequence[_U]]:
+    results = list_unzip(zip_chain(iterable))
+    return results
+
 def sort_on(index: int, seq: Sequence[Tuple[_T, _U]]) -> Sequence[Tuple[_T, _U]]:
     """A function that allows you to sort a sequence on the `n-th` element of a tuple
     it contains. This function orders in descending order (high to low).
